import Controller from "swaf/Controller";
import {NextFunction, Request, Response} from "express";
import URLRedirect from "../models/URLRedirect";
import {RequireAuthMiddleware, RequireRequestAuthMiddleware} from "swaf/auth/AuthComponent";
import generateSlug from "../SlugGenerator";
import config from "config";

export default class URLRedirectController extends Controller {
    public routes(): void {
        this.get('/url/:page([0-9]+)?', this.getUrlShrinker, 'url-shrinker', RequireAuthMiddleware);
        this.get('/url/shrink-script', this.downloadLinuxScript, 'url-linux-script');
        this.post('/url/shrink', this.addUrlFrontend, 'shrink-url', RequireAuthMiddleware);
    }

    protected async getUrlShrinker(req: Request, res: Response): Promise<void> {
        const user = req.as(RequireAuthMiddleware).getUser();
        const allowedDomains = config.get<string[]>('allowed_url_domains');
        res.render('url-shrinker', {
            allowed_domains: allowedDomains,
            default_domain: allowedDomains[config.get<number>('default_url_domain_for_urls')],
            urls: await URLRedirect.paginateForUser(req, 25, user.getOrFail('id')),
        });
    }

    protected async downloadLinuxScript(req: Request, res: Response): Promise<void> {
        res.download('assets/files/shrink_url.sh', 'shrink_url.sh');
    }

    protected async addUrlFrontend(req: Request, res: Response, next: NextFunction): Promise<void> {
        req.body.type = 'url';
        await URLRedirectController.addURL(
            req,
            res,
            next,
            req.body.autogen_url === undefined && req.body.slug ?
                req.body.slug :
                await generateSlug(10),
        );
    }

    public static async addURL(
        req: Request,
        res: Response,
        next: NextFunction,
        slug?: string,
        requestAuth: boolean = false,
    ): Promise<void> {
        if (req.body.type !== 'url') return next();

        const user = (requestAuth ? req.as(RequireRequestAuthMiddleware) : req.as(RequireAuthMiddleware)).getUser();
        slug = slug || req.params.slug || req.body.slug || await generateSlug(10);
        const urlRedirect = URLRedirect.create({
            user_id: user.id,
            slug: slug,
            target_url: req.body.target_url,
        });

        await urlRedirect.save();

        const domain = req.body.url_domain || config.get<string[]>('allowed_url_domains')[config.get<number>('default_url_domain_for_urls')];
        res.format({
            json: () => res.json({
                url: urlRedirect.getURL(domain),
            }),
            text: () => res.send(urlRedirect.getURL(domain)),
            html: () => {
                req.flash('success', 'URL shrunk successfully!');
                req.flash('url', urlRedirect.getURL(domain));
                res.redirect(Controller.route('url-shrinker'));
            },
        });
    }
}
